



mtq-worker --help
IF %ERRORLEVEL% NEQ 0 exit 1
mtq-info --help
IF %ERRORLEVEL% NEQ 0 exit 1
mtq-tail --help
IF %ERRORLEVEL% NEQ 0 exit 1
mtq-scheduler --help
IF %ERRORLEVEL% NEQ 0 exit 1
mtq-ctrl --help
IF %ERRORLEVEL% NEQ 0 exit 1
exit 0
